/* Emacs style mode select   -*- C -*-
   ----------------------------------------------------------------------------
   $Id: i_ggi.c,v 1.5 1998/10/06 21:56:00 chris Exp $
   
   Copyright (C) 1993-1996 by id Software, Inc.
  
   This source is available for distribution and/or modification
   only under the terms of the DOOM Source Code License as
   published by id Software. All rights reserved.
   
   The source is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   FITNESS FOR A PARTICULAR PURPOSE. See the DOOM Source Code License
   for more details.
   
   $Log: i_ggi.c,v $
   Revision 1.5  1998/10/06 21:56:00  chris
   adapted to current GGI snapshot, to compile for old version,
   define OLD_GGI
   
   Revision 1.4  1998/08/31 21:09:52  chris
   added InitGraphLib() dummy function
   
   Revision 1.3  1998/08/31 16:16:54  chris
   removed I_ReadScreen() function (now in v_video.c)

   Revision 1.2  1998/06/15 22:28:33  chris
   Fixed crash in IPX network game: I_GetEvent was called with
   uninitialized ggivis.
   Added ModeX screen update method. Selectable via "-modex"
   command line argument. Now ModeX modes doesn't have the ugly
   "lines" effect anymore, but screen update might be a bit slower.
   
   Revision 1.1  1998/06/14 22:35:16  chris
   Initial revision
   
   
   DESCRIPTION:
   DOOM graphics stuff for GGI ( http://www.ggi-project.org)
   Maintainer: Uwe Maurer - uwe_maurer@t-online.de
   http://home.t-online.de/home/uwe_maurer
   ------------------------------------------------------------------------- 
*/
static const char
rcsid[] = "$Id: i_ggi.c,v 1.6 1998/11/06 21:56:00 jsimmons Exp $";

#include <stdlib.h>
#include <unistd.h>
#include <signal.h>
#include <ggi/ggi.h>

#include "doomstat.h"
#include "i_system.h"
#include "v_video.h"
#include "m_argv.h"
#include "d_main.h"
#include "r_draw.h"

#include "doomdef.h"

static int     lastmousex = 0;
static int     lastmousey = 0;
boolean        mousemoved = false;
int            buttonstate=0;

static ggi_visual_t ggivis=NULL;

static int     width;
static int     height;
static int     db;
static const ggi_directbuffer *dbuf1, *dbuf2;
static int     usedbuf;
static uint8  *frameptr[2] = { NULL, NULL };
static int     curframe = 0;
static int     modexrefresh=0;

/* #define KEY_DEBUG */

#ifdef KEY_DEBUG
char str[32];
char *bits(int a)
{
	int i;
	for (i=0;i<8;i++) str[16-i]=((a & (1<<i))!=0)+'0';
	str[8]=' ';
	for (i=8;i<16;i++) str[15-i]=((a & (1<<i))!=0)+'0';
	return str;
}
#endif

int key(int label, int sym)
{
	int rc=0;
#ifdef KEY_DEBUG
	char text[100];
	ggiSetGCForeground(ggivis,0);
	ggiDrawBox(ggivis,0,0,320,24);
	ggiSetGCForeground(ggivis,255);
	sprintf(text," %c", label);
	ggiPuts(ggivis,0,0,text);
	sprintf(text," %i %x %s",label,label,bits(label));
	ggiPuts(ggivis,0,8,text);
	sprintf(text," %i %x %s",code,code,bits(code));
	ggiPuts(ggivis,0,16,text);
        ggiFlush(ggivis);
#endif
	switch(label) {
	case GIIK_CtrlL:  case GIIK_CtrlR:  rc=KEY_RCTRL; 	break;
	case GIIK_ShiftL: case GIIK_ShiftR: rc=KEY_RSHIFT;	break;
	case GIIK_AltL:   case GIIK_AltR:   rc=KEY_RALT;	break;
		
	case GIIUC_Escape: rc = KEY_ESCAPE;	break;
	case GIIUC_Tab:	rc = KEY_TAB;		break;
	case GIIK_Up:	rc = KEY_UPARROW;	break;
	case GIIK_Down:	rc = KEY_DOWNARROW;	break;
	case GIIK_Left:	rc = KEY_LEFTARROW;	break;
	case GIIK_Right:rc = KEY_RIGHTARROW;	break;
	case GIIK_Enter:rc = KEY_ENTER;		break;
	case GIIK_F1:	rc = KEY_F1;		break;
	case GIIK_F2:	rc = KEY_F2;		break;
	case GIIK_F3:	rc = KEY_F3;		break;
	case GIIK_F4:	rc = KEY_F4;		break;
	case GIIK_F5:	rc = KEY_F5;		break;
	case GIIK_F6:	rc = KEY_F6;		break;
	case GIIK_F7:	rc = KEY_F7;		break;
	case GIIK_F8:	rc = KEY_F8;		break;
	case GIIK_F9:	rc = KEY_F9;		break;
	case GIIK_F10:	rc = KEY_F10;		break;
	case GIIK_F11:	rc = KEY_F11;		break;
	case GIIK_F12:	rc = KEY_F12;		break;
	case GIIK_Pause:rc = KEY_PAUSE;		break;

	default:
		if (sym < 256) {
			/* ASCII key - we want those */
			rc = sym;
			/* We want lowercase */
			if (rc >='A' && rc <='Z') rc -= ('A' - 'a');
			switch (sym) {
				/* Some special cases */
			case '+': rc = KEY_EQUALS;	break;
			default:			break;
			}
		}
	}
	return rc;
}

void I_ShutdownGraphics(void)
{
	if (ggivis != NULL) {
		if (!usedbuf) {
			free(frameptr[0]);
		}
		ggiClose(ggivis);
		ggivis = NULL;
	}
	ggiExit();
}


void I_StartFrame (void)
{
}

void I_GetEvent(void)
{
	event_t event;
	ggi_event ev;
        ggi_event_mask mask;
		
	while (1) {
		struct timeval t={0,0};
		mask = ggiEventPoll(ggivis,emAll,&t);
		if (!mask) return;
		
		ggiEventRead(ggivis,&ev,emAll);
		switch (ev.any.type)
			{
			case evKeyPress:
				event.type = ev_keydown;
				event.data1 = key(ev.key.label,ev.key.sym);
				D_PostEvent(&event);
				break;
			case evKeyRelease:
				event.type = ev_keyup;
				event.data1 = key(ev.key.label,ev.key.sym);
				D_PostEvent(&event);
				break;
			case evPtrButtonPress:
				event.type = ev_mouse;
				buttonstate = event.data1 = 
					buttonstate | 1<<(ev.pbutton.button-1);
				event.data2 = event.data3 = 0;
				D_PostEvent(&event);
				break;
			case evPtrButtonRelease:
				event.type = ev_mouse;
				buttonstate = event.data1 = 
					buttonstate ^ 1<<(ev.pbutton.button-1);
				event.data2 = event.data3 = 0;
				D_PostEvent(&event);
				break;
			case evPtrAbsolute:
				event.type = ev_mouse;
				event.data1 = buttonstate;
				event.data2 = (ev.pmove.x - lastmousex) << 2;
				event.data3 = (lastmousey - ev.pmove.y) << 2;
				
				if (event.data2 || event.data3) {
					lastmousex = ev.pmove.x;
					lastmousey = ev.pmove.y;
					if (ev.pmove.x != screenwidth/2 &&
					    ev.pmove.y != screenheight/2) {
						D_PostEvent(&event);
						mousemoved = false;
					} else {
						mousemoved = true;
					}
				}
				break;
			case evPtrRelative:
				event.type = ev_mouse;
				event.data1 = buttonstate;
				event.data2 = ev.pmove.x << 2;
				event.data3 = -ev.pmove.y << 2;
				
				if (event.data2 || event.data3) {
					lastmousex += ev.pmove.x;
					lastmousey += ev.pmove.y;
					if (ev.pmove.x != screenwidth/2 &&
					    ev.pmove.y != screenheight/2) {
						D_PostEvent(&event);
						mousemoved = false;
					} else {
						mousemoved = true;
					}
				}
				break;
			}
	}
}

void I_StartTic (void)
{
	/* only call if GGI is initialized */
	if (ggivis) {
		I_GetEvent();
	} // if
}

void I_UpdateNoBlit (void)
{
}

void I_FinishUpdate (void)
{
	static int	lasttic;
	int		tics;
	int		i;
	
	// draws little dots on the bottom of the screen
	if (devparm) {
		i = I_GetTime();
		tics = i - lasttic;
		lasttic = i;
		if (tics > 20) tics = 20;
		
		for (i=0 ; i<tics*2 ; i+=2)
			screens[0][ (screenheight-1)*screenwidth+ i] = 0xff;
		for ( ; i<20*2 ; i+=2)
			screens[0][ (screenheight-1)*screenwidth + i] = 0x0;
		
	}
	if (!usedbuf) {
		if (!modexrefresh) {
			/* faster, but ugly in ModeX modes */
			ggiPutBox(ggivis,0,0,width,height,screens[0]);
		} else {
			/* slower, but nicer in ModeX modes */
			unsigned char *buf=screens[0];
			for (i=0; i<height; i++) {
				ggiPutHLine(ggivis,0,i,width,buf);
				buf += width;
			}
		}
	}
	if (db) {
		ggiSetDisplayFrame(ggivis, curframe);
		curframe = !curframe;
		screens[0] = frameptr[curframe];
		/* This might be needed for palette ops */
		ggiSetWriteFrame(ggivis, curframe);
	}


	ggiFlush(ggivis);
	I_GetEvent();
}

void I_SetPalette (byte* pal)
{
	int i;
	ggi_color col[256];
	byte *gamma=gammatable[usegamma];
	
	for (i=0;i<256;i++)
		{
		col[i].r=gamma[(*pal++)] << (GGI_COLOR_PRECISION-8);
		col[i].g=gamma[(*pal++)] << (GGI_COLOR_PRECISION-8);
		col[i].b=gamma[(*pal++)] << (GGI_COLOR_PRECISION-8);
	}
	ggiSetPalette(ggivis,0,256,col);
}

void I_CheckRes()
{
        // nothing to do...
}

void InitGraphLib(void)
{
    /* dummy here */
}

void I_InitGraphics(void)
{
	/* Change when we support more modes than 8bpp */
	int bytes_per_pixel = 1;

        fprintf (stderr,"I_InitGraphics: Init GGI-visual.\n");

	if (ggiInit() < 0) I_Error("Unable to init LibGGI!\n");
	if ((ggivis = ggiOpen(NULL)) == NULL) {
		I_Error("Unable to open default visual!\n");
	}
     	ggiSetFlags(ggivis, GGIFLAG_ASYNC);
	ggiSetEventMask(ggivis, emKey | emPointer);

	/* The signal stuff is NOT neccesary for LibGGI - at all, it is
	   just so that doom can save it's config file */
	signal(SIGINT,  (void (*)(int)) I_Quit);
	signal(SIGTERM, (void (*)(int)) I_Quit);
	signal(SIGHUP,  (void (*)(int)) I_Quit);
	signal(SIGQUIT, (void (*)(int)) I_Quit);

        modexrefresh = M_CheckParm("-modex") > 0;
	db = M_CheckParm("-doublebuffer") > 0;
	width = screenwidth;
	height = screenheight;


	if (!db || ggiSetSimpleMode(ggivis, width, height, 2, GT_8BIT) < 0) {
		if (ggiSetSimpleMode(ggivis, width, height, GGI_AUTO,
				     GT_8BIT) < 0) {
			I_Error("Can't set %ix%ix8bpp mode\n",
				width, height, db ? 2 : 1);
		}
	}

	usedbuf = 0;
	if ((dbuf1 = ggiDBGetBuffer(ggivis, 0)) != NULL &&
	    (dbuf1->type & GGI_DB_SIMPLE_PLB) &&
	    /* The stupid engine doesn't seem to support stride :( */
	    dbuf1->buffer.plb.stride*bytes_per_pixel == width &&
	    (db ? ((dbuf2 = ggiDBGetBuffer(ggivis, 1)) != NULL &&
		   (dbuf2->type & GGI_DB_SIMPLE_PLB) &&
		   dbuf2->buffer.plb.stride*bytes_per_pixel == width)
	     : 1)) {
		usedbuf = 1;
		frameptr[0] = dbuf1->write;
		fprintf(stderr, "Using DirectBuffer with");
		if (db)	{
			frameptr[1] = dbuf2->write;
			fprintf(stderr, " doublebuffering\n");
		} else {
			fprintf(stderr, " singlebuffering\n");
		}
	} else {
		if ((frameptr[0] = malloc(bytes_per_pixel*width*height))
		    == NULL) {
			I_Error("Unable to allocate memory?!\n");
		}
		/* No need for two buffers here */
		frameptr[1] = frameptr[0];
		fprintf(stderr, "Drawing into offscreen memory\n");
	}
	/* We will start drawing to frame 0, and start displaying frame 1 */
	if (db)	{
		ggiSetWriteFrame(ggivis, 0);
		ggiSetDisplayFrame(ggivis, 1);
	}

	screens[0] = frameptr[0];
	curframe = 0;
}

/*
 * Local variables:
 * c-file-style: "cpg"
 * c-basic-offset: 8
 * End:
 */
